@@ -10,7 +10,6 @@ import typing
 from collections import defaultdict
 from io import BufferedReader, BytesIO
 from os import SEEK_CUR, SEEK_SET
-from typing import Union
 
 import archinfo
 from sortedcontainers import SortedKeyList
@@ -20,6 +19,7 @@ from cle.backends.macho.binding import BindingHelper, MachOPointerRelocation, Ma
 from cle.backends.regions import Regions
 from cle.errors import CLECompatibilityError, CLEInvalidBinaryError, CLEOperationError
 
+from .encrypted_sentinel_backer import CryptSentinel
 from .macho_enums import LoadCommands as LC
 from .macho_enums import MachoFiletype, MH_flags
 from .section import MachOSection
@@ -244,6 +244,10 @@ class MachO(Backend):
             log.info("Parsing binding bytecode stream")
             self.do_binding()
 
+    def set_arch(self, arch):
+        super().set_arch(arch)
+        self.memory = CryptSentinel(arch=arch)
+
     @property
     def min_addr(self):
         return self.mapped_base
@@ -296,7 +300,10 @@ class MachO(Backend):
                 self._load_lc_data_in_code(binary_file, offset)
             elif cmd in [LC.LC_ENCRYPTION_INFO, LC.LC_ENCRYPTION_INFO_64]:  # LC_ENCRYPTION_INFO(_64)
                 log.debug("Found LC_ENCRYPTION_INFO @ %#x", offset)
-                # self._assert_unencrypted(binary_file, offset)
+                # Store the offset and size of the encrypted section for later
+                (_, _, cryptoff, cryptsize, cryptid) = self._unpack("5I", binary_file, offset, 20)
+                self.memory.set_crypt_info(cryptid, cryptoff, cryptsize)
+
             elif cmd in [LC.LC_DYLD_CHAINED_FIXUPS]:
                 log.info("Found LC_DYLD_CHAINED_FIXUPS @ %#x", offset)
                 (_, _, dataoff, datasize) = self._unpack("4I", binary_file, offset, 16)
@@ -331,6 +338,10 @@ class MachO(Backend):
                 f"Assertion triggered: {count} < {self.ncmds} or {offset - lc_offset} < {self.sizeofcmds}"
             )
 
+    @staticmethod
+    def extract_soname(path):
+        return path.split("/")[-1]
+
     @classmethod
     def is_compatible(cls, stream):
         stream.seek(0)
@@ -473,7 +484,7 @@ class MachO(Backend):
         """
         log.debug("Parsing exports")
         blob = self.export_blob
-        if blob is None:
+        if blob is None or len(blob) == 0:
             log.debug("Parsing exports done: No exports found")
             return
 
@@ -586,13 +597,6 @@ class MachO(Backend):
 
         log.debug("Done parsing data in code")
 
-    def _assert_unencrypted(self, f, off):
-        log.debug("Asserting unencrypted file")
-        (_, _, _, _, cryptid) = self._unpack("5I", f, off, 20)
-        if cryptid > 0:
-            log.error("Cannot load encrypted files")
-            raise CLEInvalidBinaryError()
-
     def _load_lc_function_starts(self, f, off):
         # note that the logic below is based on Apple's dyldinfo.cpp, no official docs seem to exist
         log.debug("Parsing function starts")
@@ -861,7 +865,7 @@ class MachO(Backend):
         # Store segment
         self.segments.append(seg)
 
-    S = typing.TypeVar("S", bound=Union[ctypes.Structure, ctypes.Union])
+    S = typing.TypeVar("S", bound=ctypes.Structure | ctypes.Union)
 
     def _get_struct(self, struct_type: type[S], offset: int) -> S:
         data = self._read(self._binary_stream, offset, ctypes.sizeof(struct_type))
@@ -877,9 +881,12 @@ class MachO(Backend):
         """
         end = -1
         buffer = b""
+        chunk_size = 1024
         while end == -1:
-            buffer += self._read(self._binary_stream, start, 1024)
+            buffer += self._read(self._binary_stream, start, chunk_size)
             end = buffer.find(b"\x00")
+            if end == -1:
+                start += chunk_size
             if max_length is not None and len(buffer) > max_length:
                 raise ValueError(f"Symbol name exceeds {max_length} bytes, giving up")
         return buffer[:end]
